#include "opencv2/opencv.hpp"
#include "math.h"

int
main()
{
    const int dispsize = 512;
    const int max_radius = 250;
    const int center = dispsize / 2;
    int v_percent = 100;

    cv::Mat hsv(dispsize, dispsize, CV_32FC3);
    cv::Mat disp;
    cv::namedWindow("disp", CV_WINDOW_AUTOSIZE);
    cv::createTrackbar("Value [%]", "disp", &v_percent, 100);

    while (1) {
        float v = v_percent / 100.0f;
        float radius = max_radius * v;

        for (int j = 0; j < dispsize; j++) {
            for (int i = 0; i < dispsize; i++) {
                float px = (float)(i - center);
                float py = - (float)(j - center);

                float h = (float)(atan2(py, px) * 180.0f / CV_PI);
                float s = sqrt(px * px + py * py) / radius;

                if (s < 1.0) {
                    hsv.at<cv::Vec3f>(j, i)[0] = h;
                    hsv.at<cv::Vec3f>(j, i)[1] = s;
                    hsv.at<cv::Vec3f>(j, i)[2] = v;
                } else {
                    hsv.at<cv::Vec3f>(j, i)[0] = 0.0f;
                    hsv.at<cv::Vec3f>(j, i)[1] = 0.0f;
                    hsv.at<cv::Vec3f>(j, i)[2] = 1.0f;
                }
            }
        }

        cv::cvtColor(hsv, disp, CV_HSV2BGR);
        cv::imshow("disp", disp);

        if (cv::waitKey(30) > 0) {
            break;  
        }
    }

    return 0;
}

