#include "opencv2/opencv.hpp"

typedef int (*logical_op)(int, int);

int logical_and(int a, int b)
{
    return a && b;
}

int logical_or(int a, int b)
{
    return a || b;
}

void morph8u(const cv::Mat& input, cv::Mat& output, logical_op op)
{
    // XXX: output must be pre-allocated and not identical to input

    const int width = input.cols;
    const int height = input.rows;

    for (int i = 0; i < width; i++) { // ignore first row
        output.at<uchar>(0, i) = 0;
    }
    
    for (int j = 1; j < height - 1; j++) {
        output.at<uchar>(j, 0) = 0; // ignore leftmost pixel
        for (int i = 1; i < width - 1; i++) {
            int result = input.at<uchar>(j, i);
            result = op(result, input.at<uchar>(j, i - 1));
            result = op(result, input.at<uchar>(j, i + 1));
            result = op(result, input.at<uchar>(j - 1, i));
            result = op(result, input.at<uchar>(j + 1, i));
            output.at<uchar>(j, i) = 255 * result;
        }
        output.at<uchar>(j, width - 1) = 0; // ignore rightmost pixel
    }
    for (int i = 0; i < width; i++) { // ignore last row
        output.at<uchar>(height - 1, i) = 0;
    }
}

void dilate8u(const cv::Mat& input, cv::Mat& output)
{
    morph8u(input, output, logical_or);
}

void erode8u(const cv::Mat& input, cv::Mat& output)
{
    morph8u(input, output, logical_and);
}

int main()
{
    cv::namedWindow("input", cv::WINDOW_AUTOSIZE);
    cv::namedWindow("output", cv::WINDOW_AUTOSIZE);

    cv::VideoCapture cap(0);
    cv::Mat input;
    cap >> input;
    cv::Mat output(input.rows, input.cols, CV_8U);
    cv::Mat work(input.rows, input.cols, CV_8U);
    cv::Mat work2(input.rows, input.cols, CV_8U);

    int thresh = 150;
    cv::createTrackbar("threshold", "input", &thresh, 255);

    while (1) {
        cap >> input;
        cv::cvtColor(input, input, CV_BGR2GRAY);
        cv::threshold(input, input, thresh, 255, cv::THRESH_BINARY);

        erode8u(input, work);
        erode8u(work, work2);
        erode8u(work2, work);

        dilate8u(work, work2);
        dilate8u(work2, work);
        dilate8u(work, output);

        cv::imshow("input", input);
        cv::imshow("output", output);
        if (cv::waitKey(30) > 0) {
            break;
        }
    }
}
